package com.novaordis.thread194629.distrib;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.distexec.DefaultExecutorService;
import org.infinispan.distexec.DistributedExecutorService;
import org.infinispan.manager.EmbeddedCacheManager;
import org.junit.Assert;
import org.junit.Test;

/**
 * @author <a href="mailto:ovidiu@novaordis.com">Ovidiu Feodorov</a>
 *
 * Copyright 2012 Nova Ordis LLC
 */
public class ConcurrentWriteReadDistribTestMasterTest extends Assert
{
    private static final Logger log = Logger.getLogger(ConcurrentWriteReadDistribTestMasterTest.class);
    private static final String SOURCE_CHILD_NODE_PREFIX         = "ChildNode";
	public static final int CACHE_ENTRY_COUNT = 5000;
	public static final boolean SINGLE_THREADED_DIST_WRITE = true;

//    public static void main(String [] args)
//    {
//      Thread t = new Thread(new ConcurrentWriteReadDistribTestMaster());
//      t.start();
//    }

    public void testConcurrentWriteRead() throws Exception
    {
        final EmbeddedCacheManager cm = CacheManagerHolder.getInstance().getCacheManager();

        Cache<Object, Object> c = cm.getCache(CacheManagerHolder.CACHE_MON);
        Cache<Object, Object> sc = cm.getCache(CacheManagerHolder.CACHE_SRC);
        Cache<Object, Object> dc = cm.getCache(CacheManagerHolder.CACHE_DEST);

        // no need to start() the cache, creation implicitly starts
        waitTillCacheIsInitialized(c);
        waitTillCacheIsInitialized(sc);
        waitTillCacheIsInitialized(dc);
        
        c.put(CacheManagerHolder.MSG_KEY, CacheManagerHolder.MSG_WORKING);
        
        final List<String> pathToProcess = populateSourceCache(sc);

        List<Future<Long>> f = submitDistCallToMoveFromSourceToDestination(sc,
				dc, pathToProcess);

        checkIfNoOfEntriesProcessedAreCorrect(f);
        
        c.put(CacheManagerHolder.MSG_KEY, CacheManagerHolder.MSG_DONE);
        
        checkIfDestinationCacheHasAllKeys(dc, pathToProcess);
    }

	private List<String> populateSourceCache(Cache<Object, Object> sc) {
		final List<String> pathToProcess = new ArrayList<String>();
        for (int i = 0; i < CACHE_ENTRY_COUNT; i++)
        {
          final String childNodeName = SOURCE_CHILD_NODE_PREFIX
              + Integer.toString(i);
          sc.put(childNodeName, childNodeName);
          pathToProcess.add(childNodeName);
        }
		return pathToProcess;
	}

	private List<Future<Long>> submitDistCallToMoveFromSourceToDestination(
			final Cache<Object, Object> sc, final Cache<Object, Object> dc,
			final List<String> pathToProcess) {
		final DistributedExecutorService des  = new DefaultExecutorService(sc);
        final Object[] keys = pathToProcess.toArray();
        return des.submitEverywhere(new RWDistributable( sc.getName(), dc.getName(), pathToProcess),keys);
	}

	private void checkIfNoOfEntriesProcessedAreCorrect(
			List<Future<Long>> f)
			throws InterruptedException, ExecutionException, Exception {
		int numPathsProcessed = 0;
        for (final Future<Long> futureResult : f)
        {
          	numPathsProcessed += futureResult.get().longValue();
        }
        if (numPathsProcessed != CACHE_ENTRY_COUNT)
        {
          throw new Exception("totalChildPathsProcessed should be "
              + CACHE_ENTRY_COUNT + " but instead is " + numPathsProcessed);
        }
	}

	private void waitTillCacheIsInitialized(Cache<Object, Object> sc)
			throws InterruptedException {
		while (!sc.getCacheManager().getStatus().allowInvocations())
        {
            log.info("sleeping, waiting for the cache to start ...");
            Thread.sleep(500l);
        }
	}

	private void checkIfDestinationCacheHasAllKeys(final Cache<Object, Object> dc,
			final List<String> pathToProcess) throws Exception {
		for (final String sourceChildPath : pathToProcess)
        {
          final Object value = dc.get(sourceChildPath);
          
          if (value == null)
          {
            throw new Exception(dc.getName()+" Destination cache lacked a key for "
                + sourceChildPath);
          }

          if (!value.equals(sourceChildPath))
          {
            throw new Exception(
                dc.getName()+" Destination cache had the wrong value for key "
                    + sourceChildPath + ", value was " + value.toString());
          }
          
          log.info(dc.getName()+" Found good key " + sourceChildPath + " in destination root node ");
        }
	}

    @Test
    public void run() throws Exception
    {
      System.out.print("Running Master");

          try
          {
            Thread.sleep(15000);
          } catch (InterruptedException e)
          {
            e.printStackTrace();
          }
          
          try {
        	CacheManagerHolder.getInstance();  
			testConcurrentWriteRead();
          } catch (Exception e)
          {
            e.printStackTrace();
			System.exit(-1);
          }
    }
}
