package com.novaordis.thread194629.distrib;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.distexec.DistributedCallable;

public class RWDistributable implements DistributedCallable<Object, Object, Long>, Serializable
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -6556097407222721167L;
	private final List<String> pathsToProcess;
	private final String destinationCacheName;
	private final String sourceCacheName;
	private final static Logger log = Logger.getLogger(RWDistributable.class);

	public RWDistributable(String sourceCacheName, String destinationCacheName, List<String> pathsToProcess) {
		super();
		this.pathsToProcess = pathsToProcess;
		this.destinationCacheName = destinationCacheName;
		this.sourceCacheName = sourceCacheName;
	}

	@Override
	public Long call() throws Exception {
		long numPathsProcessed = 0;
		if (pathsToProcess.size() > 0)
		{
			final Cache<Object, Object> sourceCache = CacheManagerHolder.getInstance().getCacheManager().getCache(sourceCacheName);
			final Cache<Object, Object> destinationCache = CacheManagerHolder.getInstance().getCacheManager().getCache(destinationCacheName);

			if(ConcurrentWriteReadDistribTestMasterTest.SINGLE_THREADED_DIST_WRITE) {
				numPathsProcessed = testSingleThreadedWritesWithinDistCallable(sourceCache, destinationCache);
			} else {
				numPathsProcessed = testMTWritesWithinDistCallable(sourceCache, destinationCache);
			}

		}
		log.info("Inside RWDistributable paths processed = "+numPathsProcessed);
		return numPathsProcessed;
	}

	private long testSingleThreadedWritesWithinDistCallable(
			final Cache<Object, Object> sourceCache,
			final Cache<Object, Object> destinationCache) throws InterruptedException, ExecutionException {
		final ExecutorService pool = Executors.newFixedThreadPool(15);
		final List<Future<String>> futureResults = new ArrayList<Future<String>>();
		long numPathsProcessed = 0;
		for (final String pathToProcess : pathsToProcess)
		  {
		    final Callable<String> callable = new RWDistributableWritesSingleThreaded(sourceCache, pathToProcess);
		    final Future<String> futureResult = pool.submit(callable);
		    futureResults.add(futureResult);
		  }
		for (final Future<String> futureResult : futureResults)
		  {			    
			String key = futureResult.get();
	        destinationCache.put(key, key);
	        log.info("Successfully put in destinationCache (key, value):  ("+ key + ", " + key + ")");
			numPathsProcessed += 1;
		  }
		return numPathsProcessed;
	}

	private long testMTWritesWithinDistCallable(final Cache<Object, Object> sourceCache, final Cache<Object, Object> destinationCache) throws InterruptedException, ExecutionException {
		final ExecutorService pool = Executors.newFixedThreadPool(15);
		final List<Future<Long>> futureResults = new ArrayList<Future<Long>>();
		long numPathsProcessed = 0;
		for (final String pathToProcess : pathsToProcess)
		  {
		    final Callable<Long> callable = new RWDistributableWritesMultiThreaded(sourceCache, destinationCache, pathToProcess);
		    final Future<Long> futureResult = pool.submit(callable);
		    futureResults.add(futureResult);
		  }
		for (final Future<Long> futureResult : futureResults)
		  {			    
			numPathsProcessed += futureResult.get().longValue();
		  }
		return numPathsProcessed;
	}

	@Override
	public void setEnvironment(Cache<Object, Object> cache,
			Set<Object> inputKeys) {
		final Logger log = Logger.getLogger(RWDistributable.class);
		log.info("Inside RWDistributable paths keys = "+inputKeys);
		pathsToProcess.clear();
		for (final Object inputKey : inputKeys)
		{
		  final String path = (String) inputKey;
		  pathsToProcess.add(path);
		}
	}
}
